<?php

namespace Cas\Quote\Policies;

use Litepie\User\Contracts\UserPolicy;
use Cas\Quote\Models\Quote;

class QuotePolicy
{

    /**
     * Determine if the given user can view the quote.
     *
     * @param UserPolicy $user
     * @param Quote $quote
     *
     * @return bool
     */
    public function view(UserPolicy $user, Quote $quote)
    {
        if ($user->canDo('quote.quote.view') && $user->isAdmin()) {
            return true;
        }

        return $quote->user_id == user_id() && $quote->user_type == user_type();
    }

    /**
     * Determine if the given user can create a quote.
     *
     * @param UserPolicy $user
     * @param Quote $quote
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('quote.quote.create');
    }

    /**
     * Determine if the given user can update the given quote.
     *
     * @param UserPolicy $user
     * @param Quote $quote
     *
     * @return bool
     */
    public function update(UserPolicy $user, Quote $quote)
    {
        if ($user->canDo('quote.quote.edit') && $user->isAdmin()) {
            return true;
        }

        return $quote->user_id == user_id() && $quote->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given quote.
     *
     * @param UserPolicy $user
     * @param Quote $quote
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Quote $quote)
    {
        return $quote->user_id == user_id() && $quote->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given quote.
     *
     * @param UserPolicy $user
     * @param Quote $quote
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Quote $quote)
    {
        if ($user->canDo('quote.quote.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given quote.
     *
     * @param UserPolicy $user
     * @param Quote $quote
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Quote $quote)
    {
        if ($user->canDo('quote.quote.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
